classdef robot
    %ROBOT Summary of this class goes here
    %   Detailed explanation goes here
    
   properties
      x = 0;
      y = 0;
      theta = 0;
      sigma_move = 0;
      sigma_turn = 0;
      sigma_sense = 0;
   end
   
   methods

      function self = robot(varargin)
          if nargin >= 3             
            self.x = varargin{1};
            self.y = varargin{2};
            self.theta = varargin{3};
            if nargin == 6
                self.sigma_move = varargin{4};
                self.sigma_turn = varargin{5};
                self.sigma_sense = varargin{6};       
            end
          end
      end
 
      function self = set(self,varargin)
          if nargin >= 3             
            self.x = varargin{1};
            self.y = varargin{2};
            self.theta = varargin{3};
            if nargin == 6
                self.sigma_move = varargin{4};
                self.sigma_turn = varargin{5};
                self.sigma_sense = varargin{6};       
            end
          end
      end
      
      function self = set_noise(self, sm, st, ss)
         self.sigma_move = sm;
         self.sigma_turn = st;
         self.sigma_sense = ss;
      end

    function Z = sense(self)      
        global landmarks
        Z = zeros(length(landmarks),1);
        for i = 1:length(landmarks)
            dist = sqrt((self.x - landmarks(i,1))^2 + (self.y - landmarks(i,2))^2);
            dist = dist + normrnd( 0, self.sigma_sense);          
            Z(i) = dist;
        end
    end
    
    function self = move(self, turn, move)
        if move < 0
            error('ValueError, Robot cannot move backwards');
        end
        global world_size
        
        % turn, and add noise to the turning command
        thetaNew = self.theta + turn + normrnd( 0, self.sigma_turn);
        self.theta = mod(thetaNew, 2*pi);
        
        % move, and add noise to the motion command
        dist = move + normrnd( 0, self.sigma_move);
        xNew = self.x + (cos(self.theta) * dist);
        yNew = self.y + (sin(self.theta) * dist);
        self.x = mod(xNew, world_size(1));    % cyclic truncate
        self.y = mod(yNew, world_size(2));
        
%         % set particle
%         res = robot(xi, yi, orientation);
%         res.set_noise(self.sigma_move, self.sigma_turn, self.sigma_sense);       
    end
    

    function prob = measurement_prob(self, measurement)      
        % calculates how likely a measurement should be
        global landmarks
        prob = 1.0;
        for i = 1:length(landmarks)
            dist = sqrt((self.x - landmarks(i,1))^2 + (self.y - landmarks(i,2))^2);       
            prob = prob * normpdf(measurement(i), dist, self.sigma_sense);
        end
    end
    
    function printState(self)
        fprintf('[x=%.2f y=%.2f orient=%.2f]', self.x, self.y, self.theta);          
    end
    
    
   end
   
end



